#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  Traces Splitter Script  -*-                        #
#                                                                              #
#                                                                              #
#  This script splits the traces file produced by tester / instrumented        #
#  application into components-specific traces files.                          #
#                                                                              #
#                                                                              #
# Required global variables:                                                   #
#   $LOGFILENAME   : name of the log file to generate.                         #
#   $NAME          : name of test node (used as a prefix for traces files)     #
#   $ATxMODE       : use x component                                           #
#                                                                              #
################################################################################
#$Id: Split.pl,v 1.100.4.10 2010/01/13 15:34:41 rfrance Exp $
################################################################################

require $SCRIPTDIR . "rtErrors.pl";

# Kinds of output files 
$tdf = 1;
$tio = 2;
$rio = 3;
$tpf = 4;
$tqf = 5;
$undefined = 7;

@log_codes = ("ERR", "TDF", "TIO", "RIO", "TPF", "TQF");
@ext = ("err", "tdf", "tio", "rio", "tpf", "tqf");
@used = (0, 0, 0, 0, 0, 0);

$tdf_file_mode = "w";

# Return the file type matching an opcode
sub GetOpcodeProduct #($)
{
  my $opcode = shift;

  if ($opcode eq "SF" ||
      $opcode eq "CS" ||
      $opcode eq "CI" ||
      $opcode eq "LK" ||
      $opcode eq "ST" ||
      $opcode eq "NO" ||
      $opcode eq "NI" ||
      $opcode eq "DI" ||
      $opcode eq "ME" ||
      $opcode eq "MR" ||
      $opcode eq "MC" ||
      $opcode eq "SY" ||
      $opcode eq "SA" ||
      $opcode eq "AC" ||
      $opcode eq "LS" ||
      $opcode eq "LE" ||
      $opcode eq "TM" ||
      $opcode eq "TS" ||
      $opcode eq "TU" ||
      $opcode eq "OT" ||
      $opcode eq "US" ||
      $opcode eq "IN" ||
      $opcode eq "MS" ||
      $opcode eq "TX" ||
      $opcode eq "SO" ||
      $opcode eq "SI" ||
      $opcode eq "HS" ||
      $opcode eq "TI" ||
      $opcode eq "ID" ||
      $opcode eq "DB" ||
      $opcode eq "ED" ||
      $opcode eq "BA" ||
      $opcode eq "EA" ||
      $opcode eq "GL")
  {
    # we do not differenciate TSF/TDF opcode
    $used[$tdf] = 1;
    return $tdf;
  }
  elsif ($opcode eq "NT" ||
         $opcode eq "DT" ||
         $opcode eq "CO" ||
         $opcode eq "FT" ||
         $opcode eq "DC" ||
         $opcode eq "TP" ||
         $opcode eq "TA" ||
         $opcode eq "TB" ||
         $opcode eq "TE" ||
         $opcode eq "EI")
  {
    $used[$tio] = 1;
    return $tio;
  }
  elsif ($opcode eq "PU" ||
         $opcode eq "PT" ||
         $opcode eq "PS" ||
         $opcode eq "PY" ||
         $opcode eq "PE" ||
         $opcode eq "PB" ||
         $opcode eq "PF" ||
         $opcode eq "PO" ||
         $opcode eq "PX")
  {
    $used[$tpf] = 1;
    return $tpf;
  }
  elsif ($opcode eq "QU" ||
         $opcode eq "QT" ||
         $opcode eq "QD" ||
         $opcode eq "QS" ||
         $opcode eq "QF")
  {
    $used[$tqf] = 1;
    return $tqf;
  }
  else
  {
    return 0;
  }
}

# Split $traces_file
sub ExecSplit #($$)
{
  my ($traces_file, $print_warnings) = @_;

  my $product = 0;
  my $line = 0;
  my $FTline;
  # Always a warning:
  $print_warnings = 1;
  my $warning_printed = 0;
  my $rejected = "atlout.rej";
  my $text = "";

  if($ENV{ATL_SPLIT_CHECK})
  {
  my $status=0;
  my $tdpEndLine=0;
  my $covState=0;
  my $covCO=0;
  my $lineStatus=0;
      if(open (TRACES, "<$traces_file"))
      {
	  while(<TRACES>)
	  {
	    $text = $_;
	    chomp($text);
	    $line++;

	    if (/^([A-Z][A-Z])( .*)?/ &&
		($product = GetOpcodeProduct ($1)))
	    {
	      if($product == $tio)
	      {
		  if(/^T[ABPE]/) {
			if($covState!=3) { $lineStatus=2; $covState=3;}
			if($covCO){
			    $lineStatus=1 unless(/^T[ABPE]\s[0-9a-f]+\s[0-9a-f]+$/);
			} else {
			    $lineStatus=1 unless(/^T[ABPE] [0-9a-f]+$/);
			}
		  }elsif(/^FT/) {
			$lineStatus=1 unless(/FT [0-9a-f]{1,8}$/);
			$lineStatus=2 unless($covState & 1); # only 0 and 2
			$covState= 2;
		  }elsif (/^DC/){
			$lineStatus=1 unless(/DC [0-9a-f]{1,8}$/);
			$lineStatus=2 if($covState!=2);
			$covState= 3;
		  }elsif (/^NT/){
		  	$covCO=0;
		  	$lineStatus=1 unless(/^NT$|^NT\s(\"[^\"]*\")((\s[0-9a-fA-F]+)?(\s[0-9a-fA-F]+)?)$/);
			$lineStatus=2 if($covState==2);
		  	$covState=1;
		  }elsif (/^DT/){
		  	$lineStatus=1 unless(/^DT 0$|^DT\s(\"[^\"]*\")$/);
		  }elsif (/^CO/){
		  	$covCO=1;
		  }elsif (/^EI/){
				$status=1;
			print "ERROR: $_ \n";
		  }
	      }
	    }
	    elsif (/^I[0-9] /)
	    {
	    }
	    elsif (/^[A-FGHJ-ORTV-Z][0-9]+.*$/)
	    {
	      if(/^D[0-4] -/) {print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored line %1 of %2\n", "",$line,$traces_file);next;}
	    }
	    else
	    {
	      if(/^tdp end/) {$tdpEndLine=1;next;} 
	      elsif(/^tdp/) {next;} #opcode used by TDP, line suppressed
	      elsif(/^WA\s*(.).*$/){
	      	$status=1;
			print "Runtime Error: ",ATL_ERROR($1),"\n";
			next;
	      }
	      $status=1;
	      print &PerlTranslate("Split.pl", "%1:%2: unrecognized line (file %3 contains all the rejected lines)\n", "%1 is the file being split, %2 is the line number and %3 is name of the file to look into", $traces_file, $line, $rejected) if (!$warning_printed);
	      $warning_printed = 1;
	    }
	    if($lineStatus)
	    {
	    	my $errType="";
			$status=1;
			$errType="line format" if($lineStatus ==1);
			$errType="unexpected sequence" if($lineStatus ==2);
	    	print &PerlTranslate("Split.pl", "ERROR: %1 at line %2 of %3. \"%4\"\n","",$errType,$line,$traces_file,$text);
	    	$lineStatus=0;
	    }
	  }
	  close (TRACES);
	  unless($tdpEndLine)
	  {
	      $status=1;
	      print &PerlTranslate("Split.pl", "ERROR: end of Execution not found in %2\n", "",$traces_file);
	  }
      } else {
	      print &PerlTranslate("Split.pl", "unable to open the file %1\n", "", $traces_file);
	      $status=1;
      }
      if($status){
      	   print &PerlTranslate("Split.pl", "Error found in file %1  => file not split\n", "", $traces_file);
	   return $status;
      }
  }
  open (TRACES, "<$traces_file") || return 5;
  open (REJECTED, ">$rejected") || ($warning_printed = 0);
 
  while(<TRACES>)
  {
    $text = $_;
    $line++;
    
    if (/^([A-Z][A-Z])( .*)?/ &&
        ($product = GetOpcodeProduct ($1)))
    {
      if(/^FT /){
         $FTline= $_;
      }else{
	 if (/^DC /){
            print { $handles[$product] } $FTline if ($opened[$product]); 
            
         }elsif (/^DT 0$/ || /^DT \"\"$/){
            $_= "DT \"" . GetDate . "\"\n";
         
         }elsif (/^PT \"(.*)\"$/){
            $_= "PT \"$1\" \"" . GetDate . "\"\n";
         
         }elsif (/^QD 0$/ || /^QD \"\"$/){
            $_= "QD \"" . GetDate . "\"\n";
            
         }
         print { $handles[$product] } $_ if ($opened[$product]);
      }
    }
    elsif ($ATVMODE &&
           /^I[0-9] /)
    {
      if(/^(I7 [0-9a-f]+ x -)[0-9a-f]/) {$_=$1;print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored line %1\n", "",$line);}
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]); 
    }
    elsif ($ATUMODE &&
           /^[A-FGHJ-ORTV-Z][0-9]+.*$/)
    {
      if(/^D[0-4] -/) {print &PerlTranslate("Split.pl", "Execution Time cannot be negative: mesure ingnored line %1\n", "",$line);next;}
      if (/^H0$/){
          $_= "H0 " . GetDate . "\n";
      }
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]); 
    }
    elsif ($ATSMODE &&
           (/^[a-df-z][0-9]+.*$/ ||
            /^f[bt][0-9]+.*$/ ||
            /^i[c-e][0-9]+.*$/ ||
            /^sd[0-9]+.*$/ ||
            /^t[fn][0-9]+.*$/ ||
            /^ATTOL SystemTest .*$/))
    {
      if (/^h 0/) {
          $_= "h " . GetDate . "\n";
      }
      $used[$rio] = 1;
      print { $handles[$rio] } $_ if ($opened[$rio]);
    }
    elsif ($print_warnings)
    {
      if(/^tdp/) {next;} #opcode used by TDP, line suppressed
      if(/^WA\s*(.).*$/){
        print "Runtime Error: ",ATL_ERROR($1),"\n";
      	next;
      }

      print &PerlTranslate("Split.pl", "%1:%2: unrecognized line (file %3 contains all the rejected lines)\n", "%1 is the file being split, %2 is the line number and %3 is name of the file to look into", $traces_file, $line, $rejected) if (!$warning_printed);
      $warning_printed = 1;
      print REJECTED "$line:$text";
    }
  }
  close (REJECTED);
  if (!$warning_printed) {
    unlink ($rejected);
  } else {
    open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
    print  LOG "DUM $rejected\n";
    close( LOG );
  }

  close (TRACES);
  return 0;
}

sub OpenDestFile #($$$)
{
   my ($type, $name, $mode) = @_;

   $output_files[$type] = $name;
   $handles[$type] = FileHandle->new ();
   if ($mode eq "a") {
     if(($warningTooBigFile) && ((-s $name) >4500000)){ #4.5MB
        $warningTooBigFile=0;
        print &PerlTranslate("Split.pl", "\nThe result files are growing and might be difficult to show. we suggest to reset the results before the next execution (see the report viewer menu)\n\n", "");
     }
     open ($handles[$type], ">>$name") 
       or die &PerlTranslate("Split.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the destination file", $name);
   } else {
     open ($handles[$type], ">$name") 
       or die &PerlTranslate("Split.pl", "Cannot open destination file \"%1\".\n", "%1 is the name of the destination file", $name);
   }
   $opened[$type] = 1;
}

sub CloseDestFile #($)
{
   my $type = shift;

   if ($used[$type]) {
     close( $handles[$type] );
   }
}

sub SplitTraces #($)
{
   my $tracesfile = shift;
   
   print &PerlTranslate("Split.pl", "Splitting '%1' traces file...\n", "%1 is the name of the file to be split", $tracesfile);

   my $warnings = 0;
   my $filen = 0;
   my $product = 0;
   my $one_file = "";

   if ($ATOMODE || $ATKMODE || $ATTMODE && !$ATJMODE)
   {
     $filen++;
     $product = $tdf;
     $TDFFILE = $NAME.'.'.$ext[$tdf] unless($TDFFILE);
     $one_file = $TDFFILE;
   }
   if ($ATCMODE)
   {
     $filen++;
     $product = $tio;
     $ATC_TIO_NAME=$NAME.'.'.$ext[$tio] unless($ATC_TIO_NAME);
   }
   if ($ATUMODE || $ATVMODE || $ATSMODE)
   {
     $filen++;
     $product = $rio;
     if ($ATUMODE || $ATVMODE)
     {
	$RIOFILE = $NAME.'.'.$ext[$rio] unless ($RIOFILE);
     }
     elsif ($ATSMODE)
     {
	$RIOFILE = "$tracesfile.date";
     }
     $one_file = $RIOFILE;
   }
   if ($ATPMODE)
   {
     $filen++;
     $product = $tpf;
     $ATP_TPF_NAME=$NAME.'.'.$ext[$tpf] unless($ATP_TPF_NAME);
   }
   if ($ATQMODE)
   {
     $filen++;
     $product = $tqf;
     $ATQ_TQF_NAME=$NAME.'.'.$ext[$tqf] unless($ATQ_TQF_NAME);
   }
   
   #### generate LOG 
   if($SOCKET_UPLOAD)
   {
      # assuming trtpd is currently splitting
      my $waitlong;
      while (!(-f "trtpd.log") && $waitlong < 5){sleep(2);$waitlong +=2;}
      print &PerlTranslate("Split.pl", "Split not required.\n") if ( $ENV{'ATTOLSTUDIO_VERBOSE'} );
      $used[$tdf]=(-f $TDFFILE);
      $used[$rio]=(-f $RIOFILE);
      $used[$tio]=$ATCMODE;
      $used[$tpf]=$ATPMODE;
      $used[$tqf]=$ATQMODE;

      my $memsum=MemproSummary($ATP_TPF_NAME) if($used[$tpf]);
      if ($LOGFILENAME) {
        open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
        print  LOG "TDF $TDFFILE\n"      if ($used[$tdf]);
        print  LOG "TIO $ATC_TIO_NAME\n" if ($used[$tio]);
        print  LOG "RIO $RIOFILE\n"      if ($used[$rio]);
        print  LOG "TPF $ATP_TPF_NAME\n" if ($used[$tpf]);
        print  LOG "TQF $ATQ_TQF_NAME\n" if ($used[$tqf]);
        print  LOG "DUM $memsum\n" if ($memsum);
        close ( LOG );
      }
      FloatConvert($RIOFILE) if(($used[$rio]) && defined($Ini{RTRT_FLOAT_DUMP}));
      print &PerlTranslate("Split.pl", "External split done.\n");
      return 0;
   }  
   elsif ( !$ATSMODE && !$ATL_MULTI_PROCESS && (($ATTMODE || $ATJMODE) && $ATT_ON_THE_FLY ) || ($ATCMODE && $ATC_ON_THE_FLY))
   {
      # assuming gui is currently splitting
      $UIriofile=$INTDIR."/atlout.rio"; #all rio line are written in this file
      
      sleep (3) ;
      #print "wait End Mark\n";
      my $waitlong=0;
      my $fsize;
      while (!(-f $INTDIR."/atlendtest.txt") && $waitlong < 15){
      	$fsize=(-s $UIriofile);sleep(2);$waitlong +=2 if($fsize==(-s $UIriofile));#print " $waitlong.$fsize ";
      }
      unlink($INTDIR."/atlendtest.txt");
      $used[$rio]=($ATUMODE || $ATVMODE);
      if(($ATUMODE || $ATVMODE) && (-f $UIriofile)) {
          # unknown line are rejected by the UI; So now split those rejected lines
          RenameFile($UIriofile,$RIOFILE);
      }else {
      	  print &PerlTranslate("Split.pl", "Split not required.\n") if ( $ENV{'ATTOLSTUDIO_VERBOSE'} );
          $used[$rio]=0;
      }
      $used[$tio]=1 if($ATCMODE && (-f $ATC_TIO_NAME));
      $used[$tdf]=(-f $TDFFILE);# this file is written at the beginning of the application
      #The split for TPF related information is done by SplitJvmpi for Java.
      $used[$tpf]=1 if($ATPMODE && ($TDP_LANGUAGE ne "JAVA"));
      $used[$tqf]=$ATQMODE;
      my $memsum=MemproSummary($ATP_TPF_NAME) if($used[$tpf]);
      if ($LOGFILENAME) {
        open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
        print  LOG "TDF $TDFFILE\n";
        print  LOG "TIO $ATC_TIO_NAME\n" if ($used[$tio]);
        print  LOG "TPF $ATP_TPF_NAME\n" if ($used[$tpf]);
        print  LOG "RIO $RIOFILE\n"      if ($used[$rio]);
        print  LOG "TQF $ATQ_TQF_NAME\n" if ($used[$tqf]);
        print  LOG "SPT $tracesfile\n";
        print  LOG "DUM $memsum\n" if ($memsum);
        close ( LOG );
      }
      FloatConvert($RIOFILE) if(($used[$rio]) && defined($Ini{RTRT_FLOAT_DUMP}));
      print &PerlTranslate("Split.pl", "On the fly split done.\n");
      return 0; ## if studio split

   }
   elsif ($filen == 1 && ($ATOMODE || $ATKMODE))
   # Optimize for these two ones
   {
     # No use to split file : rename only
     $output_files[$product] = $one_file;
     $used[$product] = 1;
     RenameFile ($tracesfile, $output_files[$product]);
     open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n");
     print  LOG "$log_codes[$product] $output_files[$product]\n";
     close( LOG );
     print &PerlTranslate("Split.pl", "Split done.\n");
     return 0;
   }
   
   ### now Do it
   
   if ($ATOMODE || $ATKMODE || $ATJMODE || $ATTMODE)
   {
     OpenDestFile ($tdf, $TDFFILE, $tdf_file_mode);
   }

   if ($ATCMODE)
   {
     OpenDestFile ($tio, $ATC_TIO_NAME, "a");
   }

   if ($ATPMODE)
   {
     OpenDestFile ($tpf, $ATP_TPF_NAME, "a");
   }

   if ($ATQMODE)
   {
     OpenDestFile ($tqf, $ATQ_TQF_NAME, "a");
   }
   if ($ATUMODE || $ATVMODE || $ATSMODE)
   {
     OpenDestFile ($rio, $RIOFILE, "w");
   }
   my $retSplit=ExecSplit ($tracesfile, $warnings);
   for ($i = $tdf; $i < $undefined; $i++)
   {
     if ($opened[$i])
     {
       close ($handles[$i]);
     }
   }
   if ($warnings)
   {
     if (!$opened[$tdf] && $used[$tdf])
     {
       print &PerlTranslate("Split.pl", "Warning: TDF traces were ignored.\n");
     }
     if (!$opened[$tio] && $used[$tio])
     {
       print &PerlTranslate("Split.pl", "Warning: TIO traces were ignored.\n");
     }
     if (!$opened[$rio] && $used[$rio])
     {
       print &PerlTranslate("Split.pl", "Warning: RIO traces were ignored.\n");
     }
     if (!$opened[$tpf] && $used[$tpf])
     {
       print &PerlTranslate("Split.pl", "Warning: TPF traces were ignored.\n");
     }
     if (!$opened[$tqf] && $used[$tqf])
     {
       print &PerlTranslate("Split.pl", "Warning: TQF traces were ignored.\n");
     }
   }

   if ($retSplit != 0)
   {
     print &PerlTranslate("Split.pl", "Error while splitting traces file '%1'. Split aborted.\n\n", "%1 is the name of the file to be split", $tracesfile);
     return $retSplit;
   }

   open ( LOG, ">>$LOGFILENAME" ) or die &PerlTranslate("Split.pl", "Cannot open log file.\n") unless($ATSMODE);

   if ($ATUMODE || $ATVMODE || $ATSMODE)
   {
     CloseDestFile ($rio);
     print  LOG "$log_codes[$rio] $output_files[$rio]\n" unless($ATSMODE);
   }

   if ($ATOMODE || $ATKMODE || $ATJMODE || $ATTMODE)
   {
     CloseDestFile ($tdf);
     print  LOG "$log_codes[$tdf] $output_files[$tdf]\n" unless($ATSMODE);
   }

   if ($ATCMODE)
   {
     CloseDestFile ($tio);
     print  LOG "$log_codes[$tio] $output_files[$tio]\n" unless($ATSMODE);
   }

   if ($ATPMODE)
   {
     CloseDestFile ($tpf);
     unless($ATSMODE){
        print  LOG "$log_codes[$tpf] $output_files[$tpf]\n" ;
        my $memsum=MemproSummary($output_files[$tpf]);
        print  LOG "DUM $memsum\n" if($memsum);
     }
   }

   if ($ATQMODE)
   {
     CloseDestFile ($tqf);
     print  LOG "$log_codes[$tqf] $output_files[$tqf]\n" unless($ATSMODE);
   }
   if($ENV{'ATTOLSTUDIO_VERBOSE'})
   {
      unlink ("$tracesfile.bak");
      rename ($tracesfile,"$tracesfile.bak");
      print  LOG "DUM $tracesfile.bak\n";
   }
   else
   {
      unlink ($tracesfile);
   }
   close( LOG ) unless($ATSMODE);
   FloatConvert($RIOFILE) if(($used[$rio]) && defined($Ini{RTRT_FLOAT_DUMP}));
   print &PerlTranslate("Split.pl", "Traces file successfully split.\n");
   return 0;
}

sub MemproSummary
{
   my $file=shift;
   my $out=$file; $out=~s/\.tpf$//;
   my %ERR;
   my $x;
   open IN, "$file" || return 0;
   while(<IN>){ 
   	if(/^PE (\w+)/){
   	   my $e=$1;
   	   unless($e=~ m/CHK|FIU/){
   	      $ERR{$e}++ ;
   	      $x++;
   	   }
   	   #print "Mempro $e, $_, $ERR{$e}\n";
   	}
   }
   close IN;
   if($x) {
      open OUT, ">$out.atp.txt";
      print OUT "---   Memory errors Summary\n";
      foreach (sort keys %ERR){
   	print OUT "$_ found= $ERR{$_} times\n";
      }
      print OUT "Total Memory errors found=$x\n";
      close OUT;
      print "$x Memory Errors found\n";
      return "$out.atp.txt";
   }
   return 0;
}

sub ParseAtsRioFile #($)
{
   my $riofile = shift;
   SplitTraces ($riofile) and return 1;
   rename ("$riofile.date", $riofile) or return 1;
   return 0;
}

sub FloatConvert #($RIOFILE)
{
    my $file=shift;
    my $exe;
    my $PATHSEP='/';
    if($^O=~/win32/i) {
    	$exe='.exe' ;
    	$PATHSEP='\\';
    }
    if(-f $file){
	if(-f "$ENV{ATLTGT}/cmd/extrdump$exe") {
		$exe=$ENV{ATLTGT}.$PATHSEP.'cmd'.$PATHSEP."extrdump".$exe 
	} else {
		$exe="extrdumpieee".$exe;
	}
	SystemP("$exe $file $file.tmp");
	rename("$file.tmp",$file);
    }
}

1;
