#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2009 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                      -*-  TDP runtime preprocessing -*-                      #
#                                                                              #
#                                                                              #
################################################################################
#$Id: adaRTprepro.pl,v 1.3.2.5 2010/01/21 18:16:26 rfrance Exp $
################################################################################

use FileHandle ;

my $directive='\#'; #@
my $CodeComment='--'; #//
my %macrosDirective;

sub includeFile
{
  my $file=shift;
  my $debug=shift;
  (my $path=$file)=~s/[^\\\/]+$//;
  my $ret=0;
  my $lineNum=0;
  my $off=0;
  my $level=0;
  if(open (my $in, $file)) {
    while (<$in>){
        $lineNum++;
	if(/^$directive/) {
    	   if(/^.define\s+(\w+)(\s+(.*))?$/){
    	   	$macrosDirective{$1}=$3 unless($off);
    	   	print OUT "$CodeComment$level $_" if($debug);
    	   }elsif(/^.include\s+\"(.+)\"/){
    	        my $f=$1;
    	   	print OUT "$CodeComment$level $_" if($debug);
    	   	$ret=includeFile($path.$f,$debug) unless($off);
    	   }elsif(/^.endif/){
    	   	$level-- if($level);
    	   	$off=0 unless($level);
    	   	print OUT "$CodeComment$level $_" if($debug);
    	   }else {
    	      if(/^.ifdef\s+(\w+)/){
    	   	$off=1 unless(defined($macrosDirective{$1}));
    	      }elsif(/^.ifndef\s+(\w+)/){
    	   	$off=1 if(defined($macrosDirective{$1}));
    	      }elsif(/^.if\s+(\w+)\s*==\s*(\w+)/){
    	   	$off=1 unless($macrosDirective{$1} eq $2);
    	   	print "Unknown macro '$1' in $file:$lineNum\n" unless(defined($macrosDirective{$1}));
    	      }elsif(/^.if\s+(\w+)\s+\!=\s+(\w+)/){
    	   	$off=1 if($macrosDirective{$1} eq $2);
    	   	print "Unknown macro '$1' in $file:$lineNum\n" unless(defined($macrosDirective{$1}));
    	      }elsif(/^.if\s+(\w+)/){
    	   	$off=1 unless($macrosDirective{$1});
    	   	print "Unknown macro '$1' in $file:$lineNum\n" unless(defined($macrosDirective{$1}));
    	      }else{
    	   	print "unknown directive; $file:$lineNum\n";
    	      }
    	      $level++ if($off);
    	      if($debug) {
		my $def='defAs' if(defined($macrosDirective{$1}));
		print "$1 $def $macrosDirective{$1} \n" ;
		print OUT "$CodeComment$level ($1 $def $macrosDirective{$1}) $_" ;
	      }
    	   }
    	} else {
	   if($off){
	      print OUT "$CodeComment",$_ if($debug);
	   }else{
	      s/^(\s*)\/[\/\*]/$1--/ if($CodeComment eq '--');
	      print OUT ;
	   }
    	}
    }
    close $in;
  } else {
  	print "unable to open the file $file\n";
  	$ret=1;
  }
  return $ret;
}

sub preprocessing
{
  my $file=shift;
  my $replace=shift;
  my $debug=shift;
  my $ret=0;
  open OUT, ">$file.tmp" or die "unable to open the file $file.tmp\n";
  $ret=includeFile($file,$debug);
  close OUT;
  if($replace){
     rename($file,$file.".bak") if($replace ==1);
     rename($file.".tmp",$file);
  }
  return $ret;
}

sub addDefine
{
  my $mac =shift;
  my $val =shift;
  if($mac=~m/=/) {
    ($mac,$val)=split(/=/,$mac);
  }
  $val=0 unless(defined($val));
  $macrosDirective{$mac}=$val;
#  print "#define $mac $val\n";
}

sub getDefine { my $mac=shift; return $macrosDirective{$mac}; }

sub helpAdaRTprepro
{
	print "$0 [-debug] [-help] [-D<define>] <files>\n";
	print "Each file enrich and modify the macroDefine list used for next file\n";
}

sub adaRTprepro
{
  my $ret=0;
  my @fileList;

  while (@ARGV) {
    $_ = shift @ARGV;
    s/\"//g;
    print "arg: $_\n" if ($debug>1);
    if (/\*/) {
      print "\nExpanding argument:\n$_\n" if ($debug>1);
      #unshift(@studio_args, "\"".$_."\"");
      $_ =~ s,\\,/,g;
      $_ =~ s/(\s)/\\$1/g;
      my @files = glob $_;
      if ($debug) {print join(',',@file), "\n";}
      unshift(@ARGV,@files);

    } elsif (/^-help$/) {
      helpAdaRTprepro();
      exit 0;
    } elsif (/^-debug$/ || /-verbose/) {
      $debug +=1;
    } elsif (/^-D(.+)$/) {
      my $def =$1;
      my ($mac,$val)=split(/=/,$def);
      $val=0 unless(defined($val));
      $macrosDirective{$mac}=$val;
      print "#define $mac $val\n" if($debug);
    } elsif (/^-/) {
      print "\nERROR: unknown option \"$_\"\n";
      helpAdaRTprepro();
      exit 1;
    } elsif(/\.(ada|ads|adb)$/ && (-f $_)) {
    	push(@fileList,$_)unless();
    } else {
    	print "rejected argument: $_\n";
    }
  }
  foreach $file (@fileList) {
	print "analyse $file\n" if($debug);
	$ret=preprocessing($file,0,$debug);
  }
  return $ret;
}

exit(adaRTprepro()) unless($main::TDPGEN);

1;
