#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2006-2011 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
# This file is a compiler wrapper. it allows insturmentation without changing any
# Makefile or batch file. 
# How it works:
# It take the compiler place and then execute the command prefixed by the attolcc. 
# the wrapping can be done by multiple ways: (see WRAPMODE var below)
# R) compiler rename
# P) put the path to the wrapper before the compiler path into PATH (That works only if the compiler is invoked without path)
# D) prefix the compiler name with the wrapper directory in the command line (eg "%ATLTGT%\cmd\CC.exe" instead of CC.exe or $COMPILER_PATH/CC.exe)
# What must be done for the R solution:
# 1) the cc_wrapper.pl must be copied next to compiler binary with the same compiler base Name <compilerName>.pl
# 2) replace _ATLTGT_ with the xdp baseName of the corresponding TDP to be used for this instrumentation
# 3) the orignal compiler must be renamed as <compilerName>_org.exe
# 4) the testrt tdpplugin.exe must be copied to replace the compiler (same name as compiler)
# 5) TestRTcl.exe "%ATLTGT%/cmd/TestRTLevel.txt" must be used to define instrumentation and activate it
# What must be done for the P solution:
# 1) this perl file must be part of the TDP Build settings with the same compiler Base name <compilerName>.pl
# 2) replace _ATLTGT_ with the xdp baseName of the corresponding TDP to be used for this instrumentation
# 3) the basic setting PERL_LAUNCHER must created and set to the <compilerName>
# 4) change the PATH environment variable to begins with $ATLTGT/cmd
# What must be done for the D solution:
# 1) same as step 1 of the previous solution
# 2) same as step 2 of the previous solution
# 3) same as step 3 of the previous solution
# 4) prefix the compiler name in the build environment with the path to $ATLTGT/cmd/
#################################################################################

use File::Copy;

$|=1;
# Obtain TestRealTime Dir
my $TestRTDir=$ENV{'TESTRTDIR'};
die "Test RealTime TESTRTDIR Environment variable not found !\n" unless($TestRTDir ne "");
$SCRIPTDIR = $ENV{'TESTRTDIR'} . "/lib/scripts/";
push(@INC,$SCRIPTDIR);

require "attolcclib.pl";

# Get TARGETDIR
$TARGETDIR=$ENV{'ATLTGT'};
$TARGETDIR=$TestRTDir.$PATH_SEPARATOR.'targets'.$PATH_SEPARATOR.'_ATLTGT_' unless($TARGETDIR =~ m/_ATLTGT_/);
my $WRAPMODE='R'; #R,P or D in accordance with the wrapping way described above.
my $shortTestRTDir=ShortPathName($TestRTDir);
$TARGETDIR=ShortPathName($TARGETDIR);
$ENV{'ATLTGT'}=$TARGETDIR;

#$INIFILE="tpcpp.ini"; # tp.ini as default uncomment for Cpp compiler only
DecodeIniFile();
my ($compilerDir,$compilerName,$e)=SplitFileName($0);
$compilerOrg=$compilerName;
if($WRAPMODE eq 'R'){
	$compilerOrg=$compilerDir.$PATH_SEPARATOR.$compilerOrg if($compilerDir);
	$compilerOrg.='_org';
} else {
	$compilerOrg=$Ini{BINDIR}.$PATH_SEPARATOR.$compilerOrg if($Ini{BINDIR});
}
$compilerOrg.='.'.$e if($e);

# Retrieve ObsTools options
my $optionFile="$TARGETDIR\\cmd\\TestRTLevel.txt";
my $optionFileAct=$optionFile."_act";
my $options="-proc -block=decision";
my $onOff="0";

if(-r "$optionFileAct")
{
	open(IN,"<$optionFileAct");
	$onOff=<IN>;
	close(IN);
}

if($onOff eq "0")
{
	my $status=system("$compilerOrg  @ARGV");
	exit($status);
}

unshift (@ARGV,$compilerOrg);
unshift (@ARGV,"--");
unshift (@ARGV,"-opt=\"$optionFile\"");
require "$TARGETDIR/cmd/attolcc.pl";
1;

