##!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2008 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#                          -*-  TDP generation -*-                             #
#                                                                              #
#                                                                              #
################################################################################
#$Id: tdpgen.pl,v 1.157.2.1.2.17 2011/03/22 14:13:07 rfrance Exp $
################################################################################

use XML::Parser;
use strict;
use File::Basename;
BEGIN {
  if ($^O eq "MSWin32") {
   require Win32::TieRegistry;
   }
}
## globals ############

my $inclDirC;
my $inclDirCPP;
our %Ini;
my $cpErr=0;
my $IOMODE;
my $FLOATMODE;
my $IO_BUFFER_SIZE;
my $NO_ATQMODE;
my $NO_ATPMODE;
my $NOC_ATQMODE;
my $NOC_ATPMODE;
my $NOP_ATQMODE;
my $NOP_ATPMODE;
my $LONGJMPMODE;
my $LibCheckNeeds=0;
my $INIT_UNSUPPORTED=0;
my @setjmpfct;
my @longjmpfct;
our @Errors;
my @plList = qw(metcc attolcc javic);
$main::GccUpdateRequired;
$main::check=0;
#######################
$main::TDPGEN=1;
die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$main::SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
require $main::SCRIPTDIR . "Translate.pl";

push(@INC,$main::SCRIPTDIR);

require "Fcts.pl";
require "version.pl";
require "adaRTprepro.pl";
EnterInScript();

$main::TESTRTDIR="$ENV{'TESTRTDIR'}";
my $targetsDir="$main::TESTRTDIR/targets";

sub usage()
{
die &PerlTranslate("tdpgen.pl", "\nUsage: tdpgen [<options>] <TdpFile> [<TargetDirectory>]\n\n");
}

my $file;
my $dirPath;
foreach my $arg (@ARGV) {
	$arg=~s/^\"(.+)\"$/$1/;
	if($arg=~/^-check$/){$main::check=1;}
	elsif($arg=~ /\.xdp$/){$file=$arg;}
	elsif(-d $arg){$dirPath=$arg;}
	else { usage();}
}

if(defined $dirPath){
    unless(defined $file){
	print "re-generating TDP\n";
	if($dirPath=~ m/^(.*)[\\\/]([^\\\/]+)$/){
		$file="$targetsDir/xml/$2.xdp";
		$dirPath=$1 if(-d $1);
		print "$file\n$dirPath\n";
	}
    }
} else {
	$dirPath=$targetsDir;
}
die &PerlTranslate("tdpgen.pl", "\nCannot find file \"%1\"\n\n", "%1 is a the missing file name", $file) unless -f $file;
die &PerlTranslate("tdpgen.pl", "\nCannot find directory \"%1\"\n\n", "%1 is a the missing directory name", $dirPath) unless -d $dirPath;
my $dirName=basename($file,".xdp");
my $ISxdpFile="XDPFILE=\"".absolutePath($file)."\"\n";
my $atltgtDir="$dirPath/$dirName";
$ENV{'ATLTGT'}="$dirPath/$dirName";

my $parser = new XML::Parser(ErrorContext => 2,
                             ParseParamEnt => 0,
                             Style=>'Subs',
                             Pkg=>'MyAnalyze'
                            );

$parser->setHandlers(Char=>\&MyAnalyze::CharHandler,Final=>\&MyAnalyze::FinalHandler);


$parser->parsefile($file);

open(FH, ">$atltgtDir/tdpGen.log");
my $old_fh = select(FH);
if (-r "$atltgtDir/cmd/postGen.pl" )
{
        require "$atltgtDir/cmd/postGen.pl";
        &postGen("$atltgtDir");
}
if($main::GccUpdateRequired)
{
        require "GccUpdate.pl";
        $ENV{'ATLTGT'}=$atltgtDir;
        $main::INIFILE="tp.ini";
        main::DecodeIniFile();
        my @args;
        my $defines = "";
        foreach (split(/,/,$Ini{'DEFAULT_DEFINES'}) ) { $defines.=" -D$_"; }

        push(@args,"-compiler=$Ini{'COMPILER'} $defines $Ini{'DEFAULT_CPPFLAGS'} $Ini{'DEFAULT_CFLAGS'}");
        push(@args,"$atltgtDir/ana/atct.def");
        push(@args,"$atltgtDir/ana/atus_c.def");
        push(@args,"$atltgtDir/ana/atus_cpp.def");
        push(@args,"$atltgtDir/ana/atl.opp");
        push(@args,"$atltgtDir/ana/atl.hpp");
        push(@args,split(/,/,$main::GccUpdateRequired));
        my $ret=GccUpdate(@args);
        unlink("$atltgtDir/tp.ini","$atltgtDir/tpcpp.ini") if($ret && $main::check);
}
close(FH);
select($old_fh);
unlink("$atltgtDir/tdpGen.log") if (-z "$atltgtDir/tdpGen.log");

exit(0);

sub atl_getRegKey
{
  my $val="";
  if ($^O eq "MSWin32") {
	my ($pathKey,$valKey) = split("::",shift);
	$pathKey=~s/\\\\/\\/g;
	my $Key = $Win32::TieRegistry::Registry->{$pathKey};
	if ($Key){
		$val = $Key->{$valKey};
	}
  }
  return $val;
}

package MyAnalyze;

use strict;
use File::Basename;
use File::Copy;
my $flag;
my $sequence; # check the token sequence into the xml
my $Name;
my $binFileName;
my $MacroName;
my $AdaMode;
my $tdpType;
my $javaMode;
my $componentTestingJava;
my $javaAssertAvailable;
my $d;

sub TDP
{
        my ($p,$data,%A) = @_;

        $flag=0;
        chdir($dirPath) || die &main::PerlTranslate("tdpgen.pl", "Cannot Change to directory %1\n", "%1 is the directory name", $dirPath);
        $Name=$A{'Name'};
	push(@Errors,&main::PerlTranslate("tdpgen.pl", "The TDP must be named\n", "")) unless ($Name); 
        createDir($dirName);
        $tdpType=$A{'type'};
        $javaMode=0;
        $javaMode=1 if(($tdpType eq "J2ME")||($tdpType eq "J2SE"));
        $AdaMode="std";
        $componentTestingJava=0;
        $javaAssertAvailable=1;
        my $os=$^O;
       	chdir($dirName);
       		createDir("cmd") if(! -d "cmd");
       		createDir("ana") if(! -d "ana");
       		createDir("lib") if(! -d "lib");
       		unlink("cmd/postGen.pl");
       		unlink("tdpGen.log");
       	chdir(".."); 
	if($tdpType eq "CCPP")
	{
         	$LibCheckNeeds=1;
                copyDir("$targetsDir/xml/c/lib","$atltgtDir/lib");
		push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot write lib files from targets/xml/c/lib. check permissions\n", "")) unless (-r "$atltgtDir/lib/TP.h"); 
	}
        if($tdpType eq "ADA")
        {
		chdir($dirName);
                chdir("ana");
                createDir("ada83");
                createDir("ada95");
       		chdir(".."); 
                chdir("lib");
                createDir("ada83");
                createDir("ada95");
       		chdir(".."); 
       		chdir(".."); 
       		
                copyDir("$targetsDir/xml/ada/ana","$atltgtDir/ana");

                openOU(">$atltgtDir/ana/ada83.alk");
                printOU("standard-ada83.ads STANDARD SPEC\n");
                printOU("system-ada83.ads SYSTEM SPEC\n");
                printOU("ada83\\ada83.alk LINK\n") if($os=~/win32/i);
                printOU("ada83/ada83.alk LINK\n") unless($os=~/win32/i);
                closeOU();
                openOU(">$atltgtDir/ana/ada95.alk");
                printOU("standard-ada95.ads STANDARD SPEC\n");
                printOU("system-ada95.ads SYSTEM SPEC\n");
                printOU("ada95\\ada95.alk LINK\n") if($os=~/win32/i);
                printOU("ada95/ada95.alk LINK\n") unless ($os=~/win32/i);
                closeOU();

                copyDir("$targetsDir/xml/ada/ana/ada83","$atltgtDir/ana/ada83");
                copyDir("$targetsDir/xml/ada/ana/ada95","$atltgtDir/ana/ada95");

                copyDir("$targetsDir/xml/ada/lib","$atltgtDir/lib");
                
                copyDir("$targetsDir/xml/ada/lib/ada83","$atltgtDir/lib/ada83");
                copyDir("$targetsDir/xml/ada/lib/ada95","$atltgtDir/lib/ada95");
		push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot write lib files from targets/xml/ada/lib. check permissions\n", "")) unless (-r "$atltgtDir/lib/attol.ads"); 
	}
}
sub TDP_
{
	my @iniFiles=('tp.ini','tpcpp.ini','ada83.ini','ada95.ini');
        my $os=$^O;
            #
            # Copy perllauncher from the bin directory when required
            #
        my $pathseparator=":";
        $pathseparator=";" if($os=~/win32/i);

        my $bindir=dirname($^X);
	my $perllauncher .="/perllauncher";
	   $perllauncher .=".exe" if($os=~/win32/i);
        my $f;
        if(dirname($^X) eq ".")
        {
        	foreach(split(/$pathseparator/,$ENV{'PATH'}))
                {
			$f=$_.$perllauncher;
			last if( -x $f);
		}
		$bindir=dirname($f) if(-x $f);
	}
	$main::BINDIR=$bindir;
	$perllauncher=$bindir."/".$perllauncher;
	#$perllauncher is now the file path to the perllauncher executable file
	# the list of the following pl script name require to have a perllauncher
	foreach(@plList)
	{
                if(-r "$atltgtDir/cmd/$_.pl")
                {
                	my $file="$atltgtDir/cmd/$_";
                	   $file.=".exe" if($os=~/win32/i);
                        copy1File($perllauncher,$file);
                        chmod(0755,$file);
                }
	}
	#change perl sub prototype to perl 5.8 format 
	foreach my $fpl ("atl_cc","atl_link","atl_exec") {
	     if(-f "$dirName/cmd/$fpl.pl") {
		open FPL, "<$dirName/cmd/$fpl.pl" or &main::PerlTranslate("tdpgen.pl","unable to open $fpl.pl\n", "");
		open FPO, ">$dirName/cmd/$fpl.tmp" or &main::PerlTranslate("tdpgen.pl","unable to write $fpl.tmp\n", "");
		while(<FPL>) {
		   s,^(sub atl_[a-z]+)(\(\W+\)),$1 \#$2,;
		   print FPO;
		} close FPO; close FPL;
		rename("$dirName/cmd/$fpl.tmp","$dirName/cmd/$fpl.pl");
	     }
	}
        if($tdpType eq "ADA")
        {
	    #remove old files
	    my $comm="$dirName/lib/attol_comm_";
	    unlink($comm."dump.adb",$comm."dump.ads",$comm."smart.adb",$comm."smart.ads",$comm."std.adb",$comm."std.ads");
	    $comm="$dirName/lib/attol_serv_";
	    unlink($comm."dump.adb",$comm."dump.ads",$comm."smart.adb",$comm."smart.ads",$comm."std.adb",$comm."std.ads");
	    $comm="$dirName/lib/ada95/attol_serv_95_";
	    unlink($comm."dump.adb",$comm."ravenscar.adb",$comm."smart.adb",$comm."std.adb");
	    $comm="$dirName/lib/ada95/attol_tag_";
	    unlink($comm."ravenscar.adb",$comm."ravenscar.ads");

            foreach my $pathdir ("$dirName/lib","$dirName/lib/ada95","$dirName/lib/ada83"){
		if(opendir(DIR1,$pathdir)){
		  while($d=readdir(DIR1)) {
			main::preprocessing($pathdir.'/'.$d,2,0) if ($d=~m/\.(ada|ads|adb)$/);
		  }
		  closedir(DIR1);
		}
	    }
            foreach my $pathdir ("$dirName/lib","$dirName/lib/ada95","$dirName/lib/ada83"){
		if(opendir(DIR1,$pathdir)){
		  while($d=readdir(DIR1)) {
			unlink($pathdir.'/'.$d) if ($d=~m/\.inc$/);
		  }
		  closedir(DIR1);
		}
	    }
	}
        #patch the ini file for unsuported feature
        if(($NO_ATQMODE) || ($NO_ATPMODE) || ($IOMODE) || ($IO_BUFFER_SIZE) || ($FLOATMODE)) {
           foreach my $ini (@iniFiles) {
             if(-f "$dirName/".$ini) {
 	      	open INI,">>$dirName/".$ini ;
        	print INI "ATQMODE=\"0\"\n" if($NO_ATQMODE && !$NOC_ATQMODE);
        	print INI "ATPMODE=\"0\"\n" if($NO_ATPMODE && !$NOC_ATPMODE);
        	print INI "RTRT_IO=\"$IOMODE\"\n" if($IOMODE);
        	print INI "RTRT_FLOAT_DUMP=\"$FLOATMODE\"\n" if($FLOATMODE);
        	print INI "RTRT_BUFFER_SIZE=\"$IO_BUFFER_SIZE\"\n" if($IO_BUFFER_SIZE);
		if($tdpType eq "ADA")
		{
			print INI "INSTRUM_OPT=\"-INIT_UNSUPPORTED\"\n" unless((main::getDefine("INIT_SUPPORTED")) || $INIT_UNSUPPORTED);
		}
        	close INI;
             }
           }
        }
	unlink("$dirName/all_ini.tmp");
        if(($tdpType eq "J2ME")||($tdpType eq "J2SE"))
        {
                chdir("$dirName/lib");
                createDir("jvmpi");
                copyDir("$targetsDir/xml/java/lib/jvmpi","$atltgtDir/lib/jvmpi");
                # Copy de lib/com/rational/testrealtime
                copyDir("$targetsDir/xml/java/lib/com/rational/testrealtime","$atltgtDir/lib/com/rational/testrealtime");
		push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot write lib files from targets/xml/java/lib/com/rational/testrealtime. check permissions\n", "")) unless (-r "$atltgtDir/lib/com/rational/testrealtime/Buffer.java"); 

                # Copy of jtest.xml
                copyDir("$targetsDir/xml/java/ana","$atltgtDir/ana");

                chdir("$dirName/lib");
                createDir("j2meunit"); chdir("j2meunit");
                        createDir("framework");
                        createDir("textui");
                        createDir("util");
                if($javaAssertAvailable==1)
                {
                        # Copy de lib/jdk1.4/j2meunit
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/j2meunit/framework","$atltgtDir/lib/j2meunit/framework");
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/j2meunit/textui","$atltgtDir/lib/j2meunit/textui");
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/j2meunit/util","$atltgtDir/lib/j2meunit/util");
                }
                else
                {
                        # Copy de lib/jdk1.3.1/j2meunit
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/j2meunit/framework","$atltgtDir/lib/j2meunit/framework");
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/j2meunit/textui","$atltgtDir/lib/j2meunit/textui");
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/j2meunit/util","$atltgtDir/lib/j2meunit/util");
                }
                chdir("$atltgtDir/test");
                createDir("j2meunit"); chdir("j2meunit");
                        createDir("framework");
                        createDir("textui");
                        createDir("util");
                if($javaAssertAvailable==1)
                {
                        # Copy de test/jdk1.4/j2meunit
                        copyDir("$targetsDir/xml/java/test/jdk1.4/j2meunit/framework","$atltgtDir/test/j2meunit/framework");
                        copyDir("$targetsDir/xml/java/test/jdk1.4/j2meunit/textui","$atltgtDir/test/j2meunit/textui");
                        copyDir("$targetsDir/xml/java/test/jdk1.4/j2meunit/util","$atltgtDir/test/j2meunit/util");
                }
                else
                {
                        # Copy de test/jdk1.3.1/j2meunit
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/j2meunit/framework","$atltgtDir/test/j2meunit/framework");
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/j2meunit/textui","$atltgtDir/test/j2meunit/textui");
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/j2meunit/util","$atltgtDir/test/j2meunit/util");
                }
        }
        if($tdpType eq "J2SE")
        {
                chdir("$atltgtDir/lib");
                createDir("junit"); chdir("junit");
                        createDir("extensions");
                        createDir("framework");
                        createDir("runner");
                        createDir("textui");
                if($javaAssertAvailable==1)
                {
                        # Copy de lib/jdk1.4/junit
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/junit/extensions","$atltgtDir/lib/junit/extensions");
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/junit/framework","$atltgtDir/lib/junit/framework");
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/junit/runner","$atltgtDir/lib/junit/runner");
                        copyDir("$targetsDir/xml/java/lib/jdk1.4/junit/textui","$atltgtDir/lib/junit/textui");
                }
                else
                {
                        # Copy de lib/jdk1.3.1/junit
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/junit/extensions","$atltgtDir/lib/junit/extensions");
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/junit/framework","$atltgtDir/lib/junit/framework");
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/junit/runner","$atltgtDir/lib/junit/runner");
                        copyDir("$targetsDir/xml/java/lib/jdk1.3.1/junit/textui","$atltgtDir/lib/junit/textui");
                }
                chdir("$atltgtDir/test");
                createDir("junit"); chdir("junit");
                        createDir("extensions");
                        createDir("framework");
                        createDir("runner");
                        createDir("textui");
                if($javaAssertAvailable==1)
                {
                        # Copy de test/jdk1.4/junit
                        copyDir("$targetsDir/xml/java/test/jdk1.4/junit/extensions","$atltgtDir/test/junit/extensions");
                        copyDir("$targetsDir/xml/java/test/jdk1.4/junit/framework","$atltgtDir/test/junit/framework");
                        copyDir("$targetsDir/xml/java/test/jdk1.4/junit/runner","$atltgtDir/test/junit/runner");
                        copyDir("$targetsDir/xml/java/test/jdk1.4/junit/textui","$atltgtDir/test/junit/textui");
                }
                else
                {
                        # Copy de test/jdk1.3.1/junit
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/junit/extensions","$atltgtDir/test/junit/extensions");
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/junit/framework","$atltgtDir/test/junit/framework");
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/junit/runner","$atltgtDir/test/junit/runner");
                        copyDir("$targetsDir/xml/java/test/jdk1.3.1/junit/textui","$atltgtDir/test/junit/textui");
                }
        }
}
sub IS_JAVA                     { openOU(">$dirName/tp.ini");printOU("NAME=\"Java $Name\"\nLANGUAGE=\"JAVA\"\nTESTRTVER=\"$main::VERSION\"\n$ISxdpFile"); }
sub IS_JAVA_                    { closeOU(); }
sub IS_ALL                      { openOU(">$dirName/all_ini.tmp");printOU("TESTRTVER=\"$main::VERSION\"\n$ISxdpFile");}
sub IS_ALL_                     { closeOU();}
sub IS_C                        { openOU(">$dirName/tmp.ini");printOU("NAME=\"C $Name\"\nLANGUAGE=\"C\"\n"); printFileOU("$dirName/all_ini.tmp");}
sub IS_C_                       
{
	closeOU();
	WriteIniFile("$dirName/tmp.ini","$dirName/tp.ini");
	$NOC_ATQMODE=1 if(defined $Ini{'ATQMODE'});
	$NOC_ATPMODE=1 if(defined $Ini{'ATPMODE'});
	my @inclDir=split(/[;,]/,$Ini{'STD_INCLUDE'});
	my $inclDir="";
	foreach (@inclDir) {
		$_ =~ s/\"//g;
		next if(/^\s*$/);
		$inclDir .= "#pragma attol incl_std=\"$_\"\n";
	}
	$inclDirC=$inclDir if ($inclDir ne "");
}
sub IS_CPP                      { openOU(">$dirName/tmp.ini"); printOU("NAME=\"C++ $Name\"\nLANGUAGE=\"CPP\"\n"); printFileOU("$dirName/all_ini.tmp");} 
sub IS_CPP_                     
{
	closeOU();
	WriteIniFile("$dirName/tmp.ini","$dirName/tpcpp.ini");
	$NOP_ATQMODE=1 if(defined $Ini{'ATQMODE'});
	$NOP_ATPMODE=1 if(defined $Ini{'ATPMODE'});
	my @inclDir=split(/[;,]/,$Ini{'STD_INCLUDE'});
	my $inclDir="";
	foreach (@inclDir) {
		$_ =~ s/\"//g;
		next if(/^\s*$/);
		$inclDir .= "--sys_include=\"$_\"\n";
	}
	$inclDirCPP=$inclDir if ($inclDir ne "");
}
sub IS_A                        { openOU(">$dirName/ada95.tmp"); printOU("NAME=\"Ada $Name\"\nLANGUAGE=\"ADA\"\n"); printFileOU("$dirName/all_ini.tmp");}  
sub IS_A_                       { closeOU(); WriteIniFile("$dirName/ada95.tmp","$dirName/ada95.ini");
				  $INIT_UNSUPPORTED=1 if($Ini{'INSTRUM_OPT'}=~m/-INIT_UNSUPPORTED/);}
sub IS_A83                      { openOU(">$dirName/ada83.tmp"); printOU("NAME=\"Ada83 $Name\"\nLANGUAGE=\"ADA83\"\n"); printFileOU("$dirName/all_ini.tmp");} 
sub IS_A83_                     { closeOU(); WriteIniFile("$dirName/ada83.tmp","$dirName/ada83.ini");
				  $INIT_UNSUPPORTED=1 if($Ini{'INSTRUM_OPT'}=~m/-INIT_UNSUPPORTED/);}
sub SRCEXT                      { printOU("SRCEXT=\""); }
sub SRCEXT_                     { printOU("\"\n"); }
sub OBJEXT                      { printOU("OBJEXT=\""); }
sub OBJEXT_                     { printOU("\"\n"); }
sub EXEEXT                      { printOU("EXEEXT=\""); }
sub EXEEXT_                     { printOU("\"\n"); }
sub ASMEXT                      { printOU("ASMEXT=\""); }
sub ASMEXT_                     { printOU("\"\n"); }
sub DLLEXT                      { printOU("DLLEXT=\""); }
sub DLLEXT_                     { printOU("\"\n"); }
sub LIBEXT                      { printOU("LIBEXT=\""); }
sub LIBEXT_                     { printOU("\"\n"); }
sub DEFAULT_INCLUDE_PATHS       { printOU("DEFAULT_INCLUDE_PATHS=\""); }
sub DEFAULT_INCLUDE_PATHS_      { printOU("\"\n"); }
sub DEFAULT_DEFINES             { printOU("DEFAULT_DEFINES=\""); }
sub DEFAULT_DEFINES_            { printOU("\"\n"); }
sub DEFAULT_CPPFLAGS            { printOU("DEFAULT_CPPFLAGS=\""); }
sub DEFAULT_CPPFLAGS_           { printOU("\"\n"); }
sub DEFAULT_CFLAGS              { printOU("DEFAULT_CFLAGS=\""); }
sub DEFAULT_CFLAGS_             { printOU("\"\n"); }
sub DEFAULT_LDFLAGS             { printOU("DEFAULT_LDFLAGS=\""); }
sub DEFAULT_LDFLAGS_            { printOU("\"\n"); }
sub DEFAULT_LIBRARY_PATHS       { printOU("DEFAULT_LIBRARY_PATHS=\""); }
sub DEFAULT_LIBRARY_PATHS_      { printOU("\"\n"); }
sub DEFAULT_LIBS                { printOU("DEFAULT_LIBS=\""); }
sub DEFAULT_LIBS_               { printOU("\"\n"); }
sub DEFAULT_ASMFLAGS            { printOU("DEFAULT_ASMFLAGS=\""); }
sub DEFAULT_ASMFLAGS_           { printOU("\"\n"); }
sub DEFAULT_TPLIB_CCFLAGS       { printOU("DEFAULT_TPLIB_CCFLAGS=\""); }
sub DEFAULT_TPLIB_CCFLAGS_      { printOU("\"\n"); }
sub DEFAULT_TPDLL_CCFLAGS       { printOU("DEFAULT_TPDLL_CCFLAGS=\""); }
sub DEFAULT_TPDLL_CCFLAGS_      { printOU("\"\n"); }
sub DEFAULT_TPLIB_LDFLAGS       { printOU("DEFAULT_TPLIB_LDFLAGS=\""); }
sub DEFAULT_TPLIB_LDFLAGS_      { printOU("\"\n"); }
sub DEFAULT_TPDLL_LDFLAGS       { printOU("DEFAULT_TPDLL_LDFLAGS=\""); }
sub DEFAULT_TPDLL_LDFLAGS_      { printOU("\"\n"); }
sub INIFIELD                    { $sequence=0; }
sub INIFIELD_                   { printOU("\"\n"); }

sub BUILD                       {  }
sub ASSEMBLER                   { openOU(">$dirName/cmd/atl_cc.pl"); }
sub ASSEMBLER_                  { printOU("\n"); closeOU(); }
sub COMPILATION
{
	if($tdpType eq "CCPP") {
		openOU(">>$dirName/cmd/atl_cc.pl");
	}
	else {
		openOU(">$dirName/cmd/atl_cc.pl");
	}
}
sub COMPILATION_                { printOU("\n"); closeOU(); }
sub PREPROCESSING               { openOU(">>$dirName/cmd/atl_cc.pl"); }
sub PREPROCESSING_              { printOU("\n"); closeOU(); }
sub LIBRARY                     { openOU(">$dirName/cmd/atl_link.pl"); }
sub LIBRARY_                    { printOU("\n"); closeOU(); }
sub LINK
{
	if($tdpType eq "CCPP") {
		openOU(">>$dirName/cmd/atl_link.pl");
	}
	else {
		openOU(">$dirName/cmd/atl_link.pl");
	}
}
sub LINK_                       { printOU("\n"); closeOU(); }
sub EXECUTE                     { openOU(">$dirName/cmd/atl_exec.pl"); }
sub EXECUTE_                    { printOU("\n"); closeOU(); }
sub EXECUTEDBG                  { openOU(">>$dirName/cmd/atl_exec.pl"); }
sub EXECUTEDBG_                 { printOU("\n"); closeOU(); }
sub TDPPERLFUNC                 { openOU(">$dirName/cmd/atl_func.pl"); }
sub TDPPERLFUNC_                { printOU("\n"); closeOU(); }
sub NEWCMDFILE                  { my ($p,$data,%A) = @_; my $n=$A{'Name'};
				  if($n =~/(.*)\/[^\/]+/) {my $d=$1; createSubDir($d);}
				  else {$n="cmd/$n";}
				  unlink("$dirName/$n");
				  openOU(">$dirName/$n") || 
				  push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot open file %1 for writing. check permissions\n", "%1 is the file", "$dirName/$n")); 
				}
sub NEWCMDFILE_                 { printOU("\n"); closeOU(); }
sub NEWBINFILE                  { my ($p,$data,%A) = @_;  my $n=$A{'Name'};
				  if($n =~/(.*)\/[^\/]+/) {my $d=$1; createSubDir($d);}
				  else {$n="cmd/$n";}
				  $binFileName="$dirName/$n"; openOU(">tmpEncFile.txt");}
sub NEWBINFILE_                 { closeOU();
                                  openOU(">$binFileName"); binmode(OU);
                                  open(InFile,"<tmpEncFile.txt");
                                  while(<InFile>) {
                                        printOU(DecodeBase64($_));
                                  }
                                  close(InFile); closeOU();
                                  unlink("tmpEncFile.txt");
                                  chmod(0755,$binFileName);}

sub PARSERCONF                  { }

sub PCATUSCPPDEF                { openOU(">$dirName/ana/atus_cpp.def"); if(!($inclDirC eq "")){printOU("$inclDirC\n");} }
sub PCATUSCPPDEF_               { printOU("\n"); closeOU(); }
sub PCATUSCDEF                  { openOU(">$dirName/ana/atus_c.def"  ); if(!($inclDirC eq "")){printOU("$inclDirC\n");} 
									printOU("#ifdef NEED_ATTOLCCP_DEFS\n");
									if($LONGJMPMODE ne "NONE") { foreach(@setjmpfct,@longjmpfct){s/\(.*$//; printOU("#pragma attol prefix_call \"RTRT_\" = \"$_\"\n");}}
									printOU("#endif /* NEED_ATTOLCCP_DEFS */\n");}
sub PCATUSCDEF_                 { printOU("\n"); closeOU(); }
sub PCATCTDEF                   { openOU(">$dirName/ana/atct.def"    ); if(!($inclDirC eq "")){printOU("$inclDirC\n");} 
									if($LONGJMPMODE ne "NONE") { foreach(@setjmpfct,@longjmpfct){s/\(.*$//; printOU("#pragma attol prefix_call \"RTRT_\" = \"$_\"\n");}}}
sub PCATCTDEF_                  { printOU("\n"); closeOU(); }
sub PCHPP                       { openOU(">$dirName/ana/atl.hpp"); 	if($LONGJMPMODE ne "NONE") { foreach(@setjmpfct,@longjmpfct){s/\(.*$//; printOU("#pragma attol prefix_call \"RTRT_\" = \"$_\"\n");}}}
sub PCHPP_                      { printOU("\n"); closeOU(); }
sub PCOPP                       { openOU(">$dirName/ana/atl.opp"); if(!($inclDirCPP eq "")){printOU("$inclDirCPP\n");} }
sub PCOPP_                      { printOU("\n"); closeOU(); }
sub STANDARDA83                 { openOU(">$dirName/ana/standard-ada83.ads"); }
sub STANDARDA83_                { printOU("\n");closeOU();}
sub STANDARDA95                 { openOU(">$dirName/ana/standard-ada95.ads"); }
sub STANDARDA95_                { printOU("\n");closeOU();}
sub SYSTEMADA83                 { openOU(">$dirName/ana/system-ada83.ads"); }
sub SYSTEMADA83_                { printOU("\n");closeOU();}
sub SYSTEMADA95                 { openOU(">$dirName/ana/system-ada95.ads"); }
sub SYSTEMADA95_                { printOU("\n");closeOU();}

sub LIBCONFC                    { openOU(">$dirName/lib/custom.h"); }
sub LIBCONFC_                   { printOU("\n"); closeOU(); }
sub LONGJMP                     { printOU("\n#if USE_ATT || USE_ATP || USE_ATQ\n"); }
sub LONGJMP_                    { printOU("\n#else /* USE_ATT || USE_ATP || USE_ATQ */\n"); 
					if($LONGJMPMODE ne "NONE") {foreach(@setjmpfct,@longjmpfct){s/\(.*$//; printOU("#define RTRT_$_ $_\n");}}
				  printOU("#endif /* USE_ATT || USE_ATP || USE_ATQ */\n");

				}
#
# Java
#
sub COMPONENTTESTING            { $componentTestingJava=1; }
sub COMPONENTTESTING_           { $componentTestingJava=0; }
sub CUSTOMJAVA                  { chdir($dirName);createDir("test") if(! -d "test");
                                  chdir("lib");
                                     createDir("com") if(! -d "com"); 
                                     chdir("com");
                                        createDir("rational") if(! -d "rational"); 
                                        chdir("rational");
                                           createDir("testrealtime") if(! -d "testrealtime");
                                  chdir("..");chdir("..");chdir("..");chdir("..");
                                  openOU(">$dirName/lib/com/rational/testrealtime/Custom.java");
                                  printOU("package com.rational.testrealtime;\n");
                                }
sub CUSTOMJAVA_                 { $javaMode=0; printOU("\n}\n"); closeOU(); }
sub JAVA_EARLYINIT              { printOU("public class Custom {\n"); }

sub CUSTOMJVMPI                 { chdir($dirName);
				     chdir("lib");
                                        createDir("jvmpi") if(! -d "jvmpi"); 
                                  chdir(".."); chdir("..");
                                  openOU(">$dirName/lib/jvmpi/jvmpi_custom.h");
                                  printOU("#define RTRT_TDP_COMPILATION 1\n");
                                  printOU("#define JVMPI_NONE 0\n");
                                  printOU("#define JVMPI_STD 1\n");
                                  printOU("#define JVMPI_USR 2\n"); }
sub BUILD_JVMPI                 { printOU("\n"); closeOU();
                                  openOU(">$dirName/lib/jvmpi/build.sh");
                                }
sub BUILD_JVMPI_                { printOU("\n"); closeOU(); 
                                  chmod(0755,"$dirName/lib/jvmpi/build.sh");
                                  copy1File("$dirName/lib/jvmpi/build.sh","$dirName/lib/jvmpi/build.bat");
                                }
sub TEXT_MACRO                  { my ($p,$data,%A)=@_;printOU("#define $A{'Name'} "); }
sub TEXT_MACRO_                 { printOU("\n"); }
sub BINARY_MACRO                { my ($p,$data,%A)=@_;
				    if($componentTestingJava==1) {
					$javaAssertAvailable=$A{'Value'} ; 
				    } else {
					printOU("#define $A{'Name'} $A{'Value'}");
					main::addDefine($A{'Name'}, $A{'Value'});
					$FLOATMODE="DUMP" if(($A{'Value'} eq '1') && ($A{'Name'} eq 'RTRT_FLOAT_DUMP'));
				    }
                                }
sub BINARY_MACRO_               { printOU("\n"); }
sub OPTIONAL_MACRO              { my ($p,$data,%A)=@_;
					if(($A{'Defined'} eq "1") && ($A{'Name'} eq "SETJUMP_FCTNAMES")) {@setjmpfct=split(/;/,$A{'Value'}); $A{'Defined'}="0";
						foreach(@setjmpfct){my $l=$_; unless($l=~s/\(.*\)//) {$_.="(env)"; s/\)$/,s\)/ if(/sig\w/);} 
						   push(@Errors,&main::PerlTranslate("tdpgen.pl", "Blanc spase is not allowed in SETJUMP_FCTNAMES\n","")) if(/\s/);
						   push(@Errors,&main::PerlTranslate("tdpgen.pl", "One parameter of the setjmp function must be named 'env'\n","")) unless(/\benv\b/);
						   push(@Errors,&main::PerlTranslate("tdpgen.pl", "setjmp functions separator is ;\n","")) if($l=~/,/);
						   printOU("#define RTRT_$_ (_atl_ext_stack_set((void*)&ATL_STACKLINK,(void*)env),$_)\n");
						}
					}
					if(($A{'Defined'} eq "1") && ($A{'Name'} eq "LONGJUMP_FCTNAMES")) {@longjmpfct=split(/;/,$A{'Value'}); $A{'Defined'}="0";
						foreach(@longjmpfct){my $l=$_; $_.="(env,ret)" unless($l=~s/\(.*\)//); 
						   push(@Errors,&main::PerlTranslate("tdpgen.pl", "Blanc spase is not allowed in LONGJUMP_FCTNAMES\n","")) if(/\s/);
						   push(@Errors,&main::PerlTranslate("tdpgen.pl", "One parameter of the longjmp function must be named 'env'\n","")) unless(/\benv\b/);
						   push(@Errors,&main::PerlTranslate("tdpgen.pl", "longjmp functions separator is ;\n","")) if($l=~/,/);
						   printOU("#define RTRT_$_ (_atl_ext_stack_jump((void*)env,2),$_)\n");
						}
					}
					if(($A{'Defined'} eq "1") && ($A{'Name'} eq "RTRT_BUFFER_SIZE")) {$IO_BUFFER_SIZE=$A{'Value'};}
					
					printOU("#define $A{'Name'} $A{'Value'}\n") if($A{'Defined'} eq "1"); }
					
sub CHOICE                      { my ($p,$data,%A)=@_;return if($javaMode);
                                  $MacroName=$A{'Name'};
                                  printOU("#define $A{'Name'} $A{'Value'}\n");
				  main::addDefine($MacroName, $A{'Value'});
                                  if($A{'Value'} eq "RTRT_NONE") {
                                  	if   ($MacroName eq "RTRT_CLOCK")   {$NO_ATQMODE=1;}
                                  	elsif($MacroName eq "RTRT_HEAP")    {$NO_ATPMODE=1;}
                                  	elsif($MacroName eq "RTRT_IO")      {$IOMODE="NONE";}
                                  	elsif($MacroName eq "RTRT_LONGJMP") {$LONGJMPMODE="NONE";}
                                  }elsif($A{'Value'} eq "DUMP") {
                                  	$FLOATMODE="DUMP";
                                  }elsif($MacroName eq "RTRT_IO") {
                                  	if($A{'Value'} ne "RTRT_STD") {$IOMODE=$A{'Value'};$IOMODE=~s|RTRT_||;}
                                  }
                                  $MacroName='' if($MacroName eq "RESTR");
                                  my $need="";
                                  $need="defined(".$MacroName."_NEEDS) &&" if($LibCheckNeeds);
                                  printOU("#if $need $MacroName==") if($MacroName ne ''); 
                                }
sub CHOICE_                     { return if($javaMode); printOU("#endif /* $MacroName */\n") if($MacroName ne ''); }
sub CHOICEDATA                  { my ($p,$data,%A)=@_; return if($javaMode); $flag=1 if($A{'Sensitive'}==1 && $MacroName ne ''); }
sub CHOICEDATA_                 { return if($javaMode); printOU("\n") if($flag==1); $flag=0; }

# Comments In Lib
sub COMPILER_SPECIFICS          { printOU("\n/* Compiler Specifics */\n\n"); }
sub PREPROCESSING_SPECIFICS     { printOU("/* PreProcessing Specifics */\n"); }
sub LINKAGE                     { printOU("/* Linkage Directives */\n"); }
sub CPP_SPECIFICS               { printOU("/* C++ Specifics */\n"); }
sub DATA_TYPES                  { printOU("/* Data Types */\n"); }
sub APPLICATION_SPECIFICS       { printOU("\n/* Application - RTOS specifics */\n\n"); }
sub SPRINTF                     { printOU("/* Sprintf Configuration */\n"); }
sub EXIT                        { printOU("/* Exit Configuration */\n"); }
sub SYSTEM_TESTING              { printOU("/* System Testing Specifics */\n"); }
sub OBSERVATIONS                { printOU("/* Observation Components Specifics */\n"); }
sub TESTING_SPECIFICS           { printOU("\n/* Testing Specifics */\n\n"); }
sub MAIN_RETURN                 { printOU("/* Main And Return Configuration */\n"); }
sub STRING_SUPPORT              { printOU("/* String Support */\n"); }
sub FLOAT_SUPPORT               { printOU("/* Floating Point Number Support */\n"); }
sub IO                          { printOU("\n/* Getting Back ASCII Values from the Target */\n\n"); }
sub HEAP                        { printOU("\n/* Adapting the memory allocation/free features */\n\n"); }
sub CLOCK                       { printOU("\n/* Adapting CLOCK features */\n\n"); }
sub THREADS                     { printOU("\n/* Adapting Thread features */\n\n");
                                  printOU("#if ATL_MULTI_THREADS==1\n"); }
sub THREADS_                    { printOU("#endif /* ATL_MULTI_THREADS==1 */\n"); }

#Ada Lib
sub LIBCONFA                    { openOU(">$dirName/lib/private_io.ads"); }

sub TYPEADA                     { my ($p,$data,%A)=@_; printOU("\tsubtype $A{'Name'} is ");}
sub TYPEADA_                    { printOU(";\n"); }
sub CONSTANTADA                 { my ($p,$data,%A)=@_; if($A{'Name'} eq "atc_nb_bit_branch") { printOU("\t$A{'Name'}:Constant Integer:=");}
                                                       else { printOU("\t$A{'Name'}:Constant:="); } }
sub CONSTANTADA_                { printOU(";\n"); }
sub CONSTANTSTRINGADA           { my ($p,$data,%A)=@_; printOU("\t$A{'Name'}:Constant String:="); }
sub CONSTANTSTRINGADA_          { printOU(";\n"); }

sub FREE_TEXT_BEFORE_FOR_TDP    { printOU("#ifdef RTRT_TDP_COMPILATION\n"); }
sub FREE_TEXT_BEFORE_FOR_TDP_   { printOU("\n#endif /* #ifdef RTRT_TDP_COMPILATION */\n"); }
sub MEMLIB_EXTENSION            { printOU("#if USE_ATP==1\n"); }
sub MEMLIB_EXTENSION_           { printOU("\n#endif /* #if USE_ATP==1 */\n"); }
sub FREE_TEXT_BEFORE_FOR_INSTR  { printOU("#if ((!defined(ATL_CPLUSPLUS_INSTRUMENTOR)) && (!defined(ATL_C_INSTRUMENTOR)))\n");}
sub FREE_TEXT_BEFORE_FOR_INSTR_ { printOU("\n#endif /* #if ((!defined(ATL_CPLUSPLUS...*/\n"); }
sub FREE_TEXT_BEFORE_FOR_INSTR_AND_GEN  { printOU("#ifndef RTRT_TDP_COMPILATION\n"); }
sub FREE_TEXT_BEFORE_FOR_INSTR_AND_GEN_ { printOU("\n#endif /* #ifndef RTRT_TDP_COMPILATION */\n"); }
sub FREE_TEXT_WITH_PRIV_IO_ADS  { }
sub FREE_TEXT_WITH_PRIV_IO_ADS_ { printOU("\npackage private_io is\n#if INIT_SUPPORTED == 0\npragma preelaborate;\n#endif\n"); }
sub FREE_TEXT_ADD_PRIV_IO_ADS   { printOU("\tsubtype priv_int is Integer_32b;\n");
                                  printOU("\tfunction priv_clock return priv_int;\n");
                                  printOU("\tprocedure priv_init(I : in integer);\n");
                                  printOU("\tprocedure priv_open(file : in out priv_file; name : in string );\n");
                                  printOU("\tprocedure priv_writeln(file : in out priv_file; line: in string);\n");
                                  printOU("\tprocedure priv_close(file : in out priv_file);\n");
                                  printOU("\tprocedure priv_date(date_sys:out string;lg_date :out integer);\n"); }
sub FREE_TEXT_ADD_PRIV_IO_ADS_  { printOU("\nend private_io;\n"); close(OU);}
sub FREE_TEXT_WITH_PRIV_IO_ADB  { openOU(">$dirName/lib/private_io.adb"); }
sub FREE_TEXT_WITH_PRIV_IO_ADB_ { printOU("\n#include \"private_io_with.inc\"\npackage body private_io is\n"); }

sub FREE_TEXT_TOP_ADD_PRIV_IO_ADB  { printOU("\n"); }
sub FREE_TEXT_TOP_ADD_PRIV_IO_ADB_ { printOU("\n\n#include \"private_io_std.inc\"\n"); }

sub FREE_TEXT_ADD_PRIV_IO_ADB   { printOU("procedure priv_date(date_sys:out string;lg_date :out integer) is\n");
                                  printOU("    begin\n      for i in date_sys'range loop\n        date_sys(i):=' ';\n");
                                  printOU("      end loop;\n      lg_date:=0;\n    end priv_date;\n"); }
sub FREE_TEXT_ADD_PRIV_IO_ADB_  { printOU("\nend private_io;\n"); 
				  close(OU);
				}
sub FILTERS_			{if($INIT_UNSUPPORTED && main::getDefine("INIT_SUPPORTED")) { printOU("#define INIT_SUPPORTED 0\n"); main::addDefine("INIT_SUPPORTED", "0") ;}}

sub KEY    { $flag=1 if($sequence ==0); }
sub KEY_   { printOU("=\""); $flag=0; $sequence=1;}
sub VALUEFCTC { printOU("#ifdef RTRT_TDP_COMPILATION\n"); }
sub VALUEFCTC_ { printOU("#endif /* #ifdef RTRT_TDP_COMPILATION */\n"); }
sub VALUEFCTH { printOU("#ifndef RTRT_TDP_COMPILATION\n"); }
sub VALUEFCTH_ { printOU("#endif /* #ifndef RTRT_TDP_COMPILATION */\n"); }
sub VALUE  { $flag=1; }
sub VALUE_ { $flag=0; }
sub BINVALUE  { $flag=1; }
sub BINVALUE_ { $flag=0; }
sub PROTO  { $flag=1; }
sub PROTO_ { printOU("\n"); $flag=0; }
sub TAIL   { printOU("\n"); $flag=1; }
sub TAIL_  { printOU("\n"); $flag=0; }

sub CharHandler { my ($p,$data)=@_; printOU($data) if($flag==1); }

sub FinalHandler        
{
    if (@Errors)
    {
    	open(FH, ">$atltgtDir/tdpGen.log");
        print STDERR @Errors;
	print FH @Errors;
	close(FH);
        exit(1);
    }
}

sub openOU()                    { my $f=shift; open(OU,$f) || push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot open file %1 for writing. check permissions\n", "%1 is the file", $f)); }
sub closeOU()                   { close(OU);}
sub printOU()                   { my $l=shift; print OU $l; }
sub createDir                   { my $d=shift; (mkdir ($d,0777) || push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot create directory %1 for writing. check permissions\n", "%1 is the directory name", $d))) if(! -d $d); }
sub createSubDir                { my $sd=shift; chdir($dirName);createDir($sd) if(! -d $sd); chdir("..");}
sub copy1File			{ my $src=shift; my $dest=shift;
					my $status = copy($src,$dest);
					if($status==0) {push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot copy the file %1 from %2. check permissions\n", "%1 is the uncopied file, %2 is the source", $dest,$src)) unless ($cpErr);$cpErr=1;}
				}
sub copyDir                     { my $src=shift; my $dest=shift;
                                  opendir(DIR,"$src");
                                  while($d=readdir(DIR))
                                  {
                                        next if(-d "$src/$d");
                                        next if($d=~/^\.$/);
                                        next if($d=~/^\.\.$/);
                                        copy1File("$src/$d","$dest/$d");
                                  }
                                  closedir(DIR);
                                }
sub printFileOU()
{
	my $file=shift; 
	if(-f $file){
		open INIALL, $file;
		while(<INIALL>){
		   my $l=$_;
		   print OU $l;
		}
		close INIALL;
	  }
}
                                
sub WriteIniFile #($$)
{
# evaluate the ini Values to translate the DEFAULT_xxx value to a value without $Ini and tr $ENV{'X'}->$X 
# STD_INCLUDE is evaluated as well to be used to patch ana files
#
        my ($t,$f)=@_;
	my $line;
        %Ini = ();
        open (INIFILE,"<$t")|| push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot open file %1\n", "%1 is the file name", $t));
        open (INIOUT, ">$f")|| push(@Errors,&main::PerlTranslate("tdpgen.pl", "Cannot open file %1 for writing. check permissions\n", "%1 is the file name", $f));
        while(<INIFILE>)
        {
                chomp;
                next if(/^\s*$/);
                next if(/^\s*#/);
                my ($Name,$Value)=split(/=/,$_,2);
                $Value=~s/^\"//;
                $Value=~s/\"$//;
                $Value=~s/\"\r$//;
                ($Name=~ m/^\w+$/) || push(@Errors,&main::PerlTranslate("tdpgen.pl", "Basic setting KEY >%1< after line %2 in %3 must be made of word chars\n", "%1 is the KEY name,%2 is the previous line, %3 is the ini file", $Name,$line,$f));
		$line=$_;
                if($Name=~/PERL_LAUNCHER/i)
                {
                
                	push(@plList,split(/[;,]/,$Value));
                }
                if($Value=~/^\$REG{\'(.+)\'}(.*)$/)
                {
			my $key=$1;
                	if($2) {
                		push(@Errors, "EOL expected after \$REG{'$key'}\n");
                	} else {
				$Value=main::atl_getRegKey($key);
				push(@Errors, "$key not found\n") if ($Value eq "");
			}
                }

		#be able to translate $ENV comming from above Ini
               	$Value=~s/\$Ini{\'(\w+)\'}/$Ini{$1}/g;                
               	my $Value2=$Value;
               	$Value2=~s/\$ENV{\'(\w+)\'}/$ENV{$1}/g;
		if($Name=~ /^ENV_(\w+)$/) {
			my $key=$1;
			if($Name=~ /^ENV_SET_IF_NOT_SET_(\w+)$/) {$ENV{$1}=$Value2 unless(defined($ENV{$1}));}
			else {$ENV{$key}=$Value2;}
		}
                elsif($Name=~/^DEFAULT_/i)
                {
                	$Value=~s/\$ENV{\'(\w+)\'}/\$$1/g;
                }
                elsif($Name eq "STD_INCLUDE")
                {
                	$Value=~s/\$ENV{\'(\w+)\'}/$ENV{$1}/g;
                }
                elsif($Name eq "GCCUPDATE")
                {
                	$main::GccUpdateRequired=$Value.' ';
                }
                $Ini{$Name}=$Value;
		$Value=~s/\"/\&#22\;/g;
                print INIOUT "$Name=\"$Value\"\n";
                push(@Errors, "The VALUE for the $Name KEY should not end with \\ please fix it\n") if($Value=~/[^\\]\\$/);
        }
        close(INIFILE);
        close(INIOUT);
        unlink($t);
}

sub DecodeBase64 #($)
{
    local($^W) = 0; # unpack("u",...) gives bogus warning in 5.00[123]

    my $str = shift;
    my $res = "";

    $str =~ tr|A-Za-z0-9+=/||cd;            # remove non-base64 chars
    if (length($str) % 4) {
        require Carp;
        Carp::carp(&main::PerlTranslate("tdpgen.pl", "Length of base64 data not a multiple of 4\n"))
    }
    $str =~ s/=+$//;                        # remove padding
    $str =~ tr|A-Za-z0-9+/| -_|;            # convert to uuencoded format
    while ($str =~ /(.{1,60})/gs) {
        my $len = chr(32 + length($1)*3/4); # compute length byte
        $res .= unpack("u", $len . $1 );    # uudecode
    }
    $res;
}

package main;
