/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.rqm.adapter.commandline;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;


import org.apache.abdera.model.Element;

import com.ibm.rqm.adapter.library.data.AbstractProcessor;
import com.ibm.rqm.adapter.library.data.AdapterConstants;
import com.ibm.rqm.adapter.library.data.AdapterException;
import com.ibm.rqm.adapter.library.data.ILogResult;
import com.ibm.rqm.adapter.library.data.NewRequester;


/**
 * The processor which takes in tasks and instructions fetched by the requester
 * from the server and handles it appropriately to the adapter specific
 * implementations of the {@link AbstractTestToolAdapter}.
 * 
 * @author amathur
 * 
 */
public class QMCommandLineProcessor extends AbstractProcessor implements AdapterConstants {

	private ILogResult logResult = null;
	private CommandLineExecutionThread cmdExeThread;
	
	public QMCommandLineProcessor(NewRequester requester) {
		super(requester);
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rqm.adapter.library.data.AbstractProcessor#processInstruction(org.apache.abdera.model.Element)
	 */
	@Override
	protected void processInstruction(Element instruction) {
		String type = instruction.getAttributeValue(INSTRUCTION_TYPE);
		String task = instruction.getAttributeValue(INSTRUCTION_TASK_ID);
		
		if (type.equalsIgnoreCase("CANCEL") && cmdExeThread.getTaskId().equals(task))
		{
			if(cmdExeThread!= null){
				
				cmdExeThread.getCommandLineAdapter().stopExecution();

			}
			
		}

	}

	
	protected void processExecution(String url, Element element){
		
	    cmdExeThread = new CommandLineExecutionThread(url, element, requester, this);
		cmdExeThread.start();
		
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.rqm.adapter.library.AbstractProcessor#processImport(java.lang.String,
	 *      java.util.Map, com.ibm.rqm.adapter.library.Requester)
	 */
	@Override
	protected void processImport(String url, Element element) {

		// Get most of the data from the propertiesMap
		String location = requester.getValue("qmtask:scriptPath", element);
		File folder = new File(location);
		if(folder.exists())
		{	
			if(folder.isFile())
			{
				try {
					requester.uploadResources(url, (new String[] {folder.getName()}),folder.getParentFile().getPath());
				} catch (AdapterException e) {
					Logger.Log.error("Could not upload the file: " + e.getMessage(), e);
				}
			}
			else
			{
				File[] files = folder.listFiles(new FileFilter() {
					public boolean accept(File pathname) {
						return (pathname.isFile());
					}
				});
				List<String> resources = new ArrayList<String>(files.length);
				for (int i = 0; i < files.length; i++) {
					resources.add(files[i].getName());
				}
				try {
					requester.uploadResources(url, resources.toArray(new String[] {}),folder.getPath());
				} catch (AdapterException e) {
					Logger.Log.error("Could not upload resources: " + e.getMessage(), e);
				}
			}
		}
		else
		{
			try {
				requester.uploadResources(url, (new String[] {}), null);
			} catch (AdapterException e) {
				Logger.Log.error("Could not upload empty array: " + e.getMessage(), e);
			}
		}
	}
	public synchronized void setBusy(boolean busy){
		
		super.setBusy(busy);
	}

}
